/*
	Copyright (c) 2001 John C. Bowman (USB support)

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "eph_usb.h"
#include "usb_camera_table.h"

extern int quiet;
int suppress_init_errors=0;

#ifdef HAVE_USB_H
static int inep=0x83;
static int outep=0x04;
#endif

#ifdef MSWINDOWS
#define IOCTL_RESET_PIPE 0x8000201c

typedef enum {
  EVENT_PIPE,
  READ_DATA_PIPE,
  WRITE_DATA_PIPE,
  ALL_PIPE
} PIPE_TYPE;

void
eph_win_reset_usb(HANDLE fd)
{
  PIPE_TYPE pipe = ALL_PIPE;
  DWORD ret;
  DeviceIoControl(fd, IOCTL_RESET_PIPE, &pipe, sizeof(pipe), NULL, 0, 
		  &ret, NULL);
} 
#endif

int
eph_open_usb(eph_iob *iob)
{
#if defined(MSWINDOWS)
  char prefix[]="\\\\.\\Usbscan";
  char devname[MAX_PATH];
  int i;
  for(i=0; i < 100; i++) {
    sprintf(devname,"%s%d",prefix,i);
    if(iob->debug) printf("Trying to open %s\n",devname);
    suppress_init_errors=!iob->debug;
    if ((usbfd=CreateFile(devname,
			  GENERIC_READ | GENERIC_WRITE,
			  0,
			  0,
			  OPEN_EXISTING,
			  0,
			  0)) != INVALID_HANDLE_VALUE) {
      if (eph_setint(iob,77,BLOCK_SIZE-6)) {
	if(iob->debug) printf("Initialization of %s failed\n",devname);
	CloseHandle(usbfd);
	continue;
      }
      suppress_init_errors=0;
      return 0;
    }
  }
  fprintf(stderr,"open_usb failed\n");
  return -1;
#else
#ifdef HAVE_USB_H
  struct usb_device *dev;
	  
  dev=eph_init_usb(iob);
  if(dev == NULL) {
    fprintf(stderr,"cannot find usb device\n");
    return -1;
  }
	  
  usbfd=usb_open(dev);
  if(usbfd == NULL) {
    fprintf(stderr,"cannot open usb device\n");
    return -1;
  }
	    
  /* Drop supervisor privilege */
  (void)seteuid(getuid());
	  
  if(usb_set_configuration(usbfd, 1)) {
    // Try one more time.
    if(usb_set_configuration(usbfd, 1)) { 
      fprintf(stderr,"cannot configure usb device\n");
      return -1;
    }
  }
	  
  if(usb_claim_interface(usbfd, 0)) {
    fprintf(stderr,"cannot claim interface\n");
    return -1;
  }
  
  (void) eph_setint(iob,77,BLOCK_SIZE-6);
  
  return 0;
#endif
  return -1;
#endif  
}

struct usb_device *
eph_init_usb(eph_iob *iob)
{
#ifdef HAVE_USB_H
  usb_init();
  usb_find_busses();
  usb_find_devices();
#endif
  return eph_find_usb_device(iob);
}

struct usb_device *
eph_find_usb_device(eph_iob *iob)
{
#ifdef HAVE_USB_H
  struct usb_bus *bus;
  struct usb_device *dev;
  int i;

  for (bus = usb_busses; bus; bus = bus->next) {
    for (dev = bus->devices; dev; dev = dev->next) {

      if(!quiet) printf("Found usb device id 0x%x by vendor 0x%x\n",dev->descriptor.idProduct,dev->descriptor.idVendor);

      for (i = 0; strlen (usb_camera_table[i].vendor_name) > 0; i++) {
	if ((dev->descriptor.idVendor == usb_camera_table[i].vendor_id) &&
	    ( (dev->descriptor.idProduct == usb_camera_table[i].product_id) || 
	      (usb_camera_table[i].product_id == -1) )) {
	  if(!quiet) printf("Found usb camera: %s %s\n", usb_camera_table[i].vendor_name, usb_camera_table[i].product_name);
	  return dev;
	}
      }

    }
  }
#endif	
  return NULL;
}

int
eph_write_usb(usb_dev_handle *dev, char *buf, int size, int timeout)
{
  int rc;
#ifdef MSWINDOWS
  DWORD nwrit;
  rc=!WriteFile(dev,buf,size,&nwrit,NULL) || (nwrit != size);
  eph_win_reset_usb(dev);
#else
  
#ifdef HAVE_USB_H
#ifdef USB_MAXENDPOINTS
  rc=(usb_bulk_write(dev,outep,buf,size,timeout) != size);
#else  
  usb_bulk_write(dev,outep,buf,size,timeout);
  rc=0;
#endif  
  rc |= usb_clear_halt(dev,outep);
#else
  rc=-1;  
#endif  
  
#endif  
  if(rc == -1) fprintf(stderr,"usb write error\n");
/*  else if(rc == 0) fprintf(stderr,"wrote %d bytes\n",size); */
  return rc;
}

int
eph_read_usb(usb_dev_handle *dev, char *buf, int size, int timeout)
{
  int rc;
#ifdef MSWINDOWS
  DWORD rlen;
  rc=!ReadFile(dev,buf,size,&rlen,NULL);
  eph_win_reset_usb(dev);
  if(rc) rc=-1;
  else rc=rlen;
#else
  
#ifdef HAVE_USB_H
  rc=usb_bulk_read(dev,inep,buf,size,timeout);
  rc |= usb_clear_halt(dev,inep);
#else
  rc=-1;
#endif
  
#endif
  if(rc == -1) fprintf(stderr,"usb read error\n");
/*  else if(rc > 0) fprintf(stderr,"read %d bytes\n",rc); */
  return rc;
}
